<!--
  ~ SPDX-FileCopyrightText: 2024 UL Research Institutes
  ~ SPDX-License-Identifier: Apache-2.0
-->

<!-- Score -->
<div
  class="flex flex-col w-full justify-center items-start gap-2 bg-gray-200 rounded-md"
>
  <!-- Blob -->
  <div class="flex flex-start items-center gap-2 p-2">
    <span class="text-2xl text-bold"> ${quantity} </span>
    <span class="text-xl text-wrap [overflow-wrap:anywhere]"> ${text} </span>
  </div>
  <!-- End Blob -->
</div>
<!-- End Score -->
